#################################################################################
#$LastChangedDate:$
#$LastChangedRevision:$
#$LastChangedBy:$
#$HeadURL:$
#$Id:$
#################################################################################

import sys
import os
import subprocess
import shutil
import time
import getopt
import re

PSP_MOUNT	= "/mnt/psp"
PSP_VID_PATH    = "/mp_root/100mnv01"
PSP_PMP_PATH	= "/psp/video"	
PSP_PIC_PATH	= "/psp/photo"	
TMP_FILE	= "./psp_video"
FFMPEG		= "/usr/bin/ffmpeg"
CONVERT		= "/usr/bin/convert"
PMP_MUX		= "pmp_muxer.pl"


def error(s):
    print "#" * 60
    print s
    print "#" * 60
    umount(0)
    sys.exit(-1)

def log(s):
    print "-" * 60
    print s
    print "-" * 60


def parse_opts():
    cmds = 'm:i:t:d:conb:'
    cmdl =  [   'mode',
		'infile',
                'title',
                'duration',
                'cleanup',
                'copy-only',
		'no-copy',
		'bitrate',
		
            ]
    opts, args = getopt.getopt(sys.argv[1:],cmds ,cmdl)
    mode	= None
    file_in	= None
    title	= "PSP Video"
    duration	= None
    cleanup     = False
    copyonly	= False
    nocopy	= False
    bitrate	= str(768)
    
    for o, a in opts:
        if o in ('-i',"--infile"):
	    file_in = a
        if o in ('-t',"--title="):
	    title = a
        if o in ('-d',"--duration"):
            duration = a
        if o in ('-c',"--cleanup"):
	    cleanup = True
        if o in ('-o',"--copy-only"):
	    copyonly = True
	if o in ('-m',"--mode"):
	    mode = a.lower()
	if o in ('-n',"--no-copy"):
	    nocopy = True
	if o in ('-b',"--bitrate"):
	    bitrate = str(int(a))
	


    return mode,file_in,title,duration,cleanup,copyonly,nocopy,bitrate
 
def mount():
    log("Try to mount '%s'" % PSP_MOUNT)
    cmd_mount = ['mount',PSP_MOUNT]
    while True:
        ret = subprocess.call(cmd_mount)
        if ret:
	    log("Mount '%s' failed, retrying" % PSP_MOUNT)
	    time.sleep(1)
	else:
	    break   

def umount(e=1):
    cmd_umount = ['umount',PSP_MOUNT]
    log("Try to umount '%s'" % PSP_MOUNT)
    ret = subprocess.call(cmd_umount)
    if ret and e:
        error("Umount '%s' failed" % PSP_MOUNT)
  
def sync():
    cmd_sync = ['sync']
    log("Try to sync ")
    ret = subprocess.call(cmd_sync)
 
  
def cleanup_files(p):
    path = "%s%s" %( PSP_MOUNT,p)
    d = os.listdir(path)
    for f in d:
	f = os.path.join(path,f)
	log("Remove %s" % f)
	os.unlink(f)
    # Hack: Otherwise newly create files don't get regonized afterwards 
    sync()
    umount()
    mount()
    

def main():
    mode,file_in,title,duration,cleanup,copyonly,nocopy,bitrate = parse_opts()
    if not mode:
	error("Unknown mode, use 'vid', 'pic' or 'pmp'")
   
    cmd_umount = ['umount',PSP_MOUNT]
    log("Try umount old ")
    ret = subprocess.call(cmd_umount)
     
    if mode == 'vid':     
	file_vid_out = "%s.mp4" % ( TMP_FILE )
	file_thm_out = "%s.thm" % ( TMP_FILE )

	if not file_in or not os.path.isfile(file_in):
	    error("Cannot open '%s'" % file_in)
 
	if not copyonly: 
	    cmd_vid = [FFMPEG,'-y','-i',file_in,'-title',title,'-bitexact','-s','320x240','-r','14.985','-b',bitrate,'-acodec','aac','-ac','2','-ar','24000','-ab','64','-f','psp']
	    if duration:
		cmd_vid.append('-t')
		cmd_vid.append(duration)
	    cmd_vid.append(file_vid_out)
    
	    cmd_thm = [FFMPEG,'-y','-i',file_in,'-f','singlejpeg','-ss','5','-vframes','1','-s','160x120','-an',file_thm_out]
	    log("Create Thumbnail") 
	    ret = subprocess.call(cmd_thm)
	    if ret:
		error("Thumbnail generation failed")
	    log("Encode video")
	    subprocess.call(cmd_vid)
	    if ret:
		error("Video encode failed")
	if not os.path.isfile(file_thm_out):
	    error("Cannot find tmp file '%s'" % file_thm_out)
	if not os.path.isfile(file_vid_out):
	    error("Cannot find tmp file '%s'" % file_vid_out)
    
	if nocopy:
	    sys.exit(0) 
	
	mount()
	
	if cleanup:
	    cleanup_files(PSP_VID_PATH)
	    
	found = False
	for i in range(1,10000):
	    file = "%s%s/M4V%05i.MP4"  % ( PSP_MOUNT,PSP_VID_PATH,i)
	    if not os.path.isfile(file):	
		found = True
		break
	if not found:
	    error("Not able to find proper traget file name")
	
	file_psp_vid = file
	file_psp_thm = "%s%s/M4V%05i.THM" % ( PSP_MOUNT,PSP_VID_PATH,i)
   
	log("Copy '%s' -> '%s'" % (file_thm_out, file_psp_thm))
	shutil.copy(file_thm_out,file_psp_thm)
	log("Copy '%s' -> '%s'" % (file_vid_out, file_psp_vid))
	shutil.copy(file_vid_out,file_psp_vid)
	sync()
	umount()
   
    if mode == 'pmp':     
	file_vid_out = "%s.avi" % ( TMP_FILE )
	file_aud_out = "%s.mp3" % ( TMP_FILE )
	file_out = re.compile("\.[\w]+$").sub("",os.path.basename(file_in)) + ".pmp"
	
	if not file_in or not os.path.isfile(file_in):
	    error("Cannot open '%s'" % file_in)
 
	if not copyonly: 
	    cmd_vid = [FFMPEG,'-y','-i',file_in,'-s','480x272','-r','20','-b',bitrate,'-an','-vcodec','xvid']
	    if duration:
		cmd_vid.append('-t')
		cmd_vid.append(duration)
	    cmd_vid.append(file_vid_out)
    
	    cmd_aud = [FFMPEG,'-y','-i',file_in,'-vn','-ar','44100','-ab','128','-acodec','mp3']
	    if duration:
		cmd_aud.append('-t')
		cmd_aud.append(duration)
	    cmd_aud.append(file_aud_out)
    

	    log("Create Audio") 
	    ret = subprocess.call(cmd_aud)
	    if ret:
		error("Audio encoding failed")
	    log("Encode video")
	    subprocess.call(cmd_vid)
	    if ret:
		error("Video encoding failed")
	if not os.path.isfile(file_aud_out):
	    error("Cannot find tmp file '%s'" % file_aud_out)
	if not os.path.isfile(file_vid_out):
	    error("Cannot find tmp file '%s'" % file_vid_out)
    
	
	cmd_mux = ['perl',PMP_MUX,file_vid_out,file_aud_out,file_out]
	ret = subprocess.call(cmd_mux)
	if ret:
	    error("Video mux  failed")
	if not os.path.isfile(file_out):
	    error("Cannot find tmp file '%s'" % file_out)
	
	os.unlink(file_aud_out)
	os.unlink(file_vid_out)

	if nocopy:
	    sys.exit(0) 
	
	mount()
	if cleanup:
	    cleanup_files(PSP_PMP_PATH)
	file_target = "%s%s/%s" % (PSP_MOUNT,PSP_PMP_PATH,file_out) 
	log("Copy '%s' -> '%s'" % (file_out, file_target))
	shutil.copy(file_out,file_target)
	sync()
	umount()
	
   
    if mode == 'pic':
	mount()
	if cleanup:
	    cleanup_files(PSP_PIC_PATH)
	
	if not file_in or not os.path.isdir(file_in):
	    error("Pictures not found in '%s'" % file_in)

	path = "%s%s" % (PSP_MOUNT,PSP_PIC_PATH)
	d = os.listdir(file_in)
	for f in d:
	    f_in   = os.path.join(file_in,f)
	    f_out  = os.path.join(path,f) 
	    cmd  = [CONVERT,"-geometry","480x272",f_in,f_out]
	    log("Convert '%s' -> '%s'" % (f_in,f_out))
	    subprocess.call(cmd)
	sync()
	umount()
	
   
if __name__ == '__main__':
    main()


